{% extends "!layout.html" %}
  {% block menu %}
    {{ super() }}
      <p class="caption"><span class="caption-text">Links</span></p>
      <ul>
        <li class="toctree-l1"><a class="reference external" href="https://github.com/{{ github_user }}/{{ github_repo }}">GitHub repository <span style="font-family: FontAwesome; font-size: smaller; margin-left: 3px;"></span></a></li>
        <li class="toctree-l1"><a class="reference external" href="https://github.com/{{ github_user }}/{{ github_repo }}/issues">Issue tracker <span style="font-family: FontAwesome; font-size: smaller; margin-left: 3px;"></span></a></li>
      </ul>
{% endblock %}

  {% block footer %} {{ super() }}

  <style>
    /* Sidebar header (and topbar for mobile) */
    .wy-side-nav-search, .wy-nav-top {
      background: #AA3939;
    }
    /* Sidebar */
    .wy-nav-side {
      background: #801515;
    }
  </style>
{% endblock %}
